@@ -18,26 +18,21 @@
 //! Defines memory-related functions, such as allocate/deallocate/reallocate memory
 //! regions, cache and allocation alignments.
 
+use std::alloc::{handle_alloc_error, Layout};
+use std::fmt::{Debug, Formatter};
 use std::mem::size_of;
+use std::panic::RefUnwindSafe;
 use std::ptr::NonNull;
-use std::{
-    alloc::{handle_alloc_error, Layout},
-    sync::atomic::AtomicIsize,
-};
-
-use crate::types::NativeType;
+use std::sync::Arc;
 
 mod alignment;
+mod types;
 
 pub use alignment::ALIGNMENT;
+pub use types::NativeType;
 
-// If this number is not zero after all objects have been `drop`, there is a memory leak
-static mut ALLOCATIONS: AtomicIsize = AtomicIsize::new(0);
-
-/// # Safety
-/// This pointer may only be used to check if memory is allocated.
 #[inline]
-pub unsafe fn dangling<T: NativeType>() -> NonNull<T> {
+unsafe fn null_pointer<T: NativeType>() -> NonNull<T> {
     NonNull::new_unchecked(ALIGNMENT as *mut T)
 }
 
@@ -47,10 +42,9 @@ pub unsafe fn dangling<T: NativeType>() -> NonNull<T> {
 pub fn allocate_aligned<T: NativeType>(size: usize) -> NonNull<T> {
     unsafe {
         if size == 0 {
-            dangling()
+            null_pointer()
         } else {
             let size = size * size_of::<T>();
-            ALLOCATIONS.fetch_add(size as isize, std::sync::atomic::Ordering::SeqCst);
 
             let layout = Layout::from_size_align_unchecked(size, ALIGNMENT);
             let raw_ptr = std::alloc::alloc(layout) as *mut T;
@@ -65,10 +59,9 @@ pub fn allocate_aligned<T: NativeType>(size: usize) -> NonNull<T> {
 pub fn allocate_aligned_zeroed<T: NativeType>(size: usize) -> NonNull<T> {
     unsafe {
         if size == 0 {
-            dangling()
+            null_pointer()
         } else {
             let size = size * size_of::<T>();
-            ALLOCATIONS.fetch_add(size as isize, std::sync::atomic::Ordering::SeqCst);
 
             let layout = Layout::from_size_align_unchecked(size, ALIGNMENT);
             let raw_ptr = std::alloc::alloc_zeroed(layout) as *mut T;
@@ -77,16 +70,17 @@ pub fn allocate_aligned_zeroed<T: NativeType>(size: usize) -> NonNull<T> {
     }
 }
 
-/// Frees memory previously allocated by [`allocate_aligned_zeroed`] or [`allocate_aligned`].
 /// # Safety
-/// This function is sound iff:
 ///
-/// * `ptr` was allocated by [`allocate_aligned_zeroed`] or [`allocate_aligned`]
-/// * `size` must be the same size that was used to allocate that block of memory.
+/// This function is unsafe because undefined behavior can result if the caller does not ensure all
+/// of the following:
+///
+/// * ptr must denote a block of memory currently allocated via this allocator,
+///
+/// * size must be the same size that was used to allocate that block of memory,
 pub unsafe fn free_aligned<T: NativeType>(ptr: NonNull<T>, size: usize) {
-    if size != 0 {
+    if ptr != null_pointer() {
         let size = size * size_of::<T>();
-        ALLOCATIONS.fetch_sub(size as isize, std::sync::atomic::Ordering::SeqCst);
         std::alloc::dealloc(
             ptr.as_ptr() as *mut u8,
             Layout::from_size_align_unchecked(size, ALIGNMENT),
@@ -94,29 +88,33 @@ pub unsafe fn free_aligned<T: NativeType>(ptr: NonNull<T>, size: usize) {
     }
 }
 
-/// Reallocates memory previously allocated by [`allocate_aligned_zeroed`] or [`allocate_aligned`].
 /// # Safety
-/// This function is sound iff `ptr` was previously allocated by `allocate_aligned` or `allocate_aligned_zeroed` for `old_size` items.
+///
+/// This function is unsafe because undefined behavior can result if the caller does not ensure all
+/// of the following:
+///
+/// * ptr must be currently allocated via this allocator,
+///
+/// * new_size must be greater than zero.
+///
+/// * new_size, when rounded up to the nearest multiple of [ALIGNMENT], must not overflow (i.e.,
+/// the rounded value must be less than usize::MAX).
 pub unsafe fn reallocate<T: NativeType>(
     ptr: NonNull<T>,
     old_size: usize,
     new_size: usize,
 ) -> NonNull<T> {
-    if old_size == 0 {
+    let old_size = old_size * size_of::<T>();
+    let new_size = new_size * size_of::<T>();
+    if ptr == null_pointer() {
         return allocate_aligned(new_size);
     }
 
     if new_size == 0 {
         free_aligned(ptr, old_size);
-        return dangling();
+        return null_pointer();
     }
-    let old_size = old_size * size_of::<T>();
-    let new_size = new_size * size_of::<T>();
 
-    ALLOCATIONS.fetch_add(
-        new_size as isize - old_size as isize,
-        std::sync::atomic::Ordering::SeqCst,
-    );
     let raw_ptr = std::alloc::realloc(
         ptr.as_ptr() as *mut u8,
         Layout::from_size_align_unchecked(old_size, ALIGNMENT),
@@ -126,3 +124,32 @@ pub unsafe fn reallocate<T: NativeType>(
         handle_alloc_error(Layout::from_size_align_unchecked(new_size, ALIGNMENT))
     })
 }
+
+/// The owner of an allocation.
+/// The trait implementation is responsible for dropping the allocations once no more references exist.
+pub trait Allocation: RefUnwindSafe {}
+
+impl<T: RefUnwindSafe> Allocation for T {}
+
+/// Mode of deallocating memory regions
+pub(crate) enum Deallocation {
+    /// An allocation of the given capacity that needs to be deallocated using arrows's cache aligned allocator.
+    /// See [allocate_aligned] and [free_aligned].
+    Arrow(usize),
+    /// An allocation from an external source like the FFI interface or a Rust Vec.
+    /// Deallocation will happen
+    Custom(Arc<dyn Allocation>),
+}
+
+impl Debug for Deallocation {
+    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+        match self {
+            Deallocation::Arrow(capacity) => {
+                write!(f, "Deallocation::Arrow {{ capacity: {} }}", capacity)
+            }
+            Deallocation::Custom(_) => {
+                write!(f, "Deallocation::Custom {{ capacity: unknown }}")
+            }
+        }
+    }
+}
